@@ -25,10 +25,21 @@ class ApplicationTest extends TestCase
         $inputMock = $this->getMock('Symfony\Component\Console\Input\InputInterface');
         $outputMock = $this->getMock('Symfony\Component\Console\Output\OutputInterface');
 
-        $inputMock->expects($this->once())
+        $index = 0;
+        $inputMock->expects($this->at($index++))
+            ->method('getParameterOption')
+            ->with($this->equalTo(array('--working-dir', '-d')))
+            ->will($this->returnValue(false));
+
+        $inputMock->expects($this->at($index++))
             ->method('getFirstArgument')
             ->will($this->returnValue('list'));
 
+        $inputMock->expects($this->at($index++))
+            ->method('hasParameterOption')
+            ->with($this->equalTo('--no-plugins'))
+            ->will($this->returnValue(true));
+
         $index = 0;
         $outputMock->expects($this->at($index++))
             ->method("writeError");
@@ -55,7 +66,6 @@ class ApplicationTest extends TestCase
             define('COMPOSER_DEV_WARNING_TIME', time() - 1);
         }
 
-        $this->setExpectedException('RuntimeException');
         $application->doRun($inputMock, $outputMock);
     }
 
@@ -68,9 +78,20 @@ class ApplicationTest extends TestCase
         $inputMock = $this->getMock('Symfony\Component\Console\Input\InputInterface');
         $outputMock = $this->getMock('Symfony\Component\Console\Output\OutputInterface');
 
-        $inputMock->expects($this->once())
+        $index = 0;
+        $inputMock->expects($this->at($index++))
+            ->method('getParameterOption')
+            ->with($this->equalTo(array('--working-dir', '-d')))
+            ->will($this->returnValue(false));
+
+        $inputMock->expects($this->at($index++))
             ->method('getFirstArgument')
-            ->will($this->returnValue($command));
+            ->will($this->returnValue('list'));
+
+        $inputMock->expects($this->at($index++))
+            ->method('hasParameterOption')
+            ->with($this->equalTo('--no-plugins'))
+            ->will($this->returnValue(true));
 
         $outputMock->expects($this->never())
             ->method("writeln");
@@ -79,7 +100,6 @@ class ApplicationTest extends TestCase
             define('COMPOSER_DEV_WARNING_TIME', time() - 1);
         }
 
-        $this->setExpectedException('RuntimeException');
         $application->doRun($inputMock, $outputMock);
     }
 
