@@ -2050,7 +2050,7 @@ class JsonManipulatorTest extends \PHPUnit_Framework_TestCase
         $manipulator = new JsonManipulator('{
     "foo": "bar"
 }');
-        
+
         $this->assertTrue($manipulator->addMainKey('bar', '$1baz'));
         $this->assertEquals('{
     "foo": "bar",
@@ -2069,7 +2069,7 @@ class JsonManipulatorTest extends \PHPUnit_Framework_TestCase
 }
 ', $manipulator->getContents());
     }
-    
+
     public function testUpdateMainKey()
     {
         $manipulator = new JsonManipulator('{
@@ -2142,7 +2142,68 @@ class JsonManipulatorTest extends \PHPUnit_Framework_TestCase
 }
 ', $manipulator->getContents());
     }
-    
+
+    public function testRemoveMainKey()
+    {
+        $manipulator = new JsonManipulator('{
+    "repositories": [
+        {
+            "package": {
+                "require": {
+                    "this/should-not-end-up-in-root-require": "~2.0"
+                },
+                "require-dev": {
+                    "this/should-not-end-up-in-root-require-dev": "~2.0"
+                }
+            }
+        }
+    ],
+    "require": {
+        "package/a": "*",
+        "package/b": "*",
+        "package/c": "*"
+    },
+    "foo": "bar",
+    "require-dev": {
+        "package/d": "*"
+    }
+}');
+
+        $this->assertTrue($manipulator->removeMainKey('repositories'));
+        $this->assertEquals('{
+    "require": {
+        "package/a": "*",
+        "package/b": "*",
+        "package/c": "*"
+    },
+    "foo": "bar",
+    "require-dev": {
+        "package/d": "*"
+    }
+}
+', $manipulator->getContents());
+
+        $this->assertTrue($manipulator->removeMainKey('foo'));
+        $this->assertEquals('{
+    "require": {
+        "package/a": "*",
+        "package/b": "*",
+        "package/c": "*"
+    },
+    "require-dev": {
+        "package/d": "*"
+    }
+}
+', $manipulator->getContents());
+
+        $this->assertTrue($manipulator->removeMainKey('require'));
+        $this->assertTrue($manipulator->removeMainKey('require-dev'));
+        $this->assertEquals('{
+}
+', $manipulator->getContents());
+
+    }
+
     public function testIndentDetection()
     {
         $manipulator = new JsonManipulator('{
